/* 2013-11 mantis 1791: */

create table FK_S_CPV_RODZAJ (ID integer not null);
alter table FK_S_CPV_RODZAJ add constraint SCPVRODZAJ_PK primary key (ID) using index SCPVRODZAJ_PK_IDX;

alter table FK_S_CPV_RODZAJ add KOD char(10) character set win1250 not null collate pxw_plk ;
alter table FK_S_CPV_RODZAJ add DATA_POCZ date default '01/01/2000' not null ;
alter table FK_S_CPV_RODZAJ add DATA_KONC date default '12/31/2100' not null ;
alter table FK_S_CPV_RODZAJ add NAZWA varchar(250) character set win1250 collate pxw_plk ;

create index SCPVRODZAJ_KODDATY_IDX on FK_S_CPV_RODZAJ (KOD,DATA_POCZ,DATA_KONC);

set term ^ ;
create or alter trigger SCPVRODZAJ_ID_TRIG for FK_S_CPV_RODZAJ
active before insert or update position 0
as
begin
  if ((new.ID is null) or (new.ID = 0)) then
    new.ID = gen_id(GEN_PB, 1);
end^
set term ; ^

/* ponizsza linia komentarza jest interpretowana przez exe jako instrukcja wykonania polecen wpisanych ponad nia: */
/*--------------------------------------------------------------- */

create table FK_S_CPV (ID integer not null);
alter table FK_S_CPV add constraint SCPV_PK primary key (ID) using index SCPV_PK_IDX;

alter table FK_S_CPV add TYP integer default 1 not null ;
alter table FK_S_CPV add KOD char(10) character set win1250 not null collate pxw_plk ;
alter table FK_S_CPV add PARENT char(10) character set win1250 collate pxw_plk ;
alter table FK_S_CPV add DATA_POCZ date default '01/01/2000' not null ;
alter table FK_S_CPV add DATA_KONC date default '12/31/2100' not null ;
alter table FK_S_CPV add NAZWA varchar(250) character set win1250 collate pxw_plk ;
alter table FK_S_CPV add ID_RODZAJ integer;

create index SCPV_KODDATY_IDX on FK_S_CPV (KOD,DATA_POCZ,DATA_KONC);
create index SCPV_TYP_IDX on FK_S_CPV (TYP);

set term ^ ;
create or alter trigger SCPV_ID_TRIG for FK_S_CPV
active before insert or update position 0
as
begin
  if ((new.ID is null) or (new.ID = 0)) then
    new.ID = gen_id(GEN_PB, 1);
end^
set term ; ^

/* ponizsza linia komentarza jest interpretowana przez exe jako instrukcja wykonania polecen wpisanych ponad nia: */
/*--------------------------------------------------------------- */

create table FK_CPV (ID integer not null);
alter table FK_CPV add constraint CPV_PK primary key (ID) using index CPV_PK_IDX;

alter table FK_CPV add ID_DOKUMENT integer not null ;
alter table FK_CPV add ID_S_CPV integer not null ;
alter table FK_CPV add KWOTA_NETTO decimal(18,4);
alter table FK_CPV add OPIS blob sub_type 0 segment size 80;

alter table FK_CPV add constraint CPV_DOK_SCPV_UNIQ unique (ID_DOKUMENT,ID_S_CPV) using index CPV_DOK_SCPV_UNIQ_IDX;

set term ^ ;
create or alter trigger CPV_PK_TRIG for FK_CPV
active before insert or update position 0
as
begin
  if ((new.ID is null) or (new.ID = 0)) then
    new.ID = gen_id(GEN_PB, 1);
end^
set term ; ^

/* ponizsza linia komentarza jest interpretowana przez exe jako instrukcja wykonania polecen wpisanych ponad nia: */
/*--------------------------------------------------------------- */

alter table FK_S_CPV add constraint SCPVRODZAJ_FK foreign key (ID_RODZAJ) references FK_S_CPV_RODZAJ(ID) on delete set null on update cascade using index SCPVRODZAJ_FK_IDX;

alter table FK_CPV add constraint CPV_DOKUMENT_FK foreign key (ID_DOKUMENT) references FK_DOKUMENTY(ID) on delete cascade on update cascade using index CPV_DOKUMENT_FK_IDX;
alter table FK_CPV add constraint CPV_SCPV_FK foreign key (ID_S_CPV) references FK_S_CPV(ID) on update cascade using index CPV_SCPV_FK_IDX;

/* ponizsza linia komentarza jest interpretowana przez exe jako instrukcja wykonania polecen wpisanych ponad nia: */
/*--------------------------------------------------------------- */

alter table FK_S_DOKUMENTY add CZY_CPV char(1) character set win1250 collate win1250 ;

/* ponizsza linia komentarza jest interpretowana przez exe jako instrukcja wykonania polecen wpisanych ponad nia: */
/*--------------------------------------------------------------- */

create or alter exception BLEDNA_KOLEJNOSC_DAT 'Data pocztkowa jest pniejsza od daty kocowej!';
set term ^ ;

create or alter trigger SCPVRODZAJ_DATY_TRIG for FK_S_CPV_RODZAJ
active before insert or update position 1
as
begin
  if (new.DATA_POCZ > new.DATA_KONC) then begin
    exception BLEDNA_KOLEJNOSC_DAT;
  end
end^

create or alter trigger SCPV_DATY_TRIG for FK_S_CPV
active before insert or update position 1
as
begin
  if (new.DATA_POCZ > new.DATA_KONC) then begin
    exception BLEDNA_KOLEJNOSC_DAT;
  end
end^

set term ; ^

/* ponizsza linia komentarza jest interpretowana przez exe jako instrukcja wykonania polecen wpisanych ponad nia: */
/*--------------------------------------------------------------- */

create or alter exception DUPLIKAT_ZAKRESU_DAT 'Duplikat okresu lub czci okresu wanoci!';
set term ^ ;

create or alter trigger SCPVRODZAJ_DUPL_TRIG_I for FK_S_CPV_RODZAJ
active before insert position 2
as
declare variable
  intCount integer;
begin
  select
    count (*)
  from
    FK_S_CPV_RODZAJ s
  where
    s.KOD = new.KOD
    and s.DATA_KONC >= new.DATA_POCZ and s.DATA_POCZ <= new.DATA_KONC
  into :intCount;
  if (intCount > 0) then exception DUPLIKAT_ZAKRESU_DAT;
end^

create or alter trigger SCPVRODZAJ_DUPL_TRIG_U for FK_S_CPV_RODZAJ
active before update position 2
as
declare variable
  intCount integer;
begin
  if ((new.KOD <> old.KOD) or (new.DATA_POCZ <> old.DATA_POCZ) or (new.DATA_KONC <> old.DATA_KONC)) then begin
    select
      count (*)
    from
      FK_S_CPV_RODZAJ s
    where
      s.ID <> old.ID
      and s.KOD = new.KOD
      and s.DATA_KONC >= new.DATA_POCZ and s.DATA_POCZ <= new.DATA_KONC
    into :intCount;
    if (intCount > 0) then exception DUPLIKAT_ZAKRESU_DAT;
  end
end^

create or alter trigger SCPV_DUPL_TRIG_I for FK_S_CPV
active before insert position 2
as
declare variable
  intCount integer;
begin
  select
    count (*)
  from
    FK_S_CPV s
  where
    s.KOD = new.KOD
    and s.DATA_KONC >= new.DATA_POCZ and s.DATA_POCZ <= new.DATA_KONC
  into :intCount;
  if (intCount > 0) then exception DUPLIKAT_ZAKRESU_DAT;
end^

create or alter trigger SCPV_DUPL_TRIG_U for FK_S_CPV
active before update position 2
as
declare variable
  intCount integer;
begin
  if ((new.KOD <> old.KOD) or (new.DATA_POCZ <> old.DATA_POCZ) or (new.DATA_KONC <> old.DATA_KONC)) then begin
    select
      count (*)
    from
      FK_S_CPV s
    where
      s.ID <> old.ID
      and s.KOD = new.KOD
      and s.DATA_KONC >= new.DATA_POCZ and s.DATA_POCZ <= new.DATA_KONC
    into :intCount;
    if (intCount > 0) then exception DUPLIKAT_ZAKRESU_DAT;
  end
end^

set term ; ^

/* ponizsza linia komentarza jest interpretowana przez exe jako instrukcja wykonania polecen wpisanych ponad nia: */
/*--------------------------------------------------------------- */

create or alter exception ZMIANA_DAT_NIEMOZLIWA 'Znaleziono dokumenty niezgodne z okresem wanoci!';
set term ^ ;

create or alter trigger SCPVRODZAJ_SCPV_TRIG for FK_S_CPV_RODZAJ
active before update position 3
as
declare variable
  intCount integer;
begin
  if ((new.DATA_POCZ <> old.DATA_POCZ) or (new.DATA_KONC <> old.DATA_KONC) or (new.ID <> old.ID)) then begin
    select
      count (*)
    from
      FK_S_CPV slocpv
    where
      slocpv.ID_RODZAJ = old.ID
      and ((new.DATA_POCZ > slocpv.DATA_KONC) or (new.DATA_KONC < slocpv.DATA_POCZ))
    into :intCount;
    if (intCount > 0) then exception ZMIANA_DAT_NIEMOZLIWA;
  end
end^

create or alter trigger SCPV_DOKUM_TRIG for FK_S_CPV
active before update position 4
as
declare variable
  intCount integer;
begin
  if ((new.DATA_POCZ <> old.DATA_POCZ) or (new.DATA_KONC <> old.DATA_KONC) or (new.ID <> old.ID)) then begin
    select
      count (*)
    from
      FK_DOKUMENTY dokum
      join FK_CPV cpvpoz on (dokum.ID = cpvpoz.ID_DOKUMENT
                             and cpvpoz.ID_S_CPV = old.ID)
    where
      dokum.DATA_DOK not between new.DATA_POCZ and new.DATA_KONC
    into :intCount;
    if (intCount > 0) then exception ZMIANA_DAT_NIEMOZLIWA;
  end
end^

set term ; ^

/* ponizsza linia komentarza jest interpretowana przez exe jako instrukcja wykonania polecen wpisanych ponad nia: */
/*--------------------------------------------------------------- */

create or alter exception NIEPRAWIDLOWY_RODZAJ_CPV 'Okres wanoci rodzaju CPV niezgodny z okresem wanoci CPV!';
set term ^ ;

create trigger SCPV_RODZ_TRIG_I for FK_S_CPV
active before insert position 3
as
declare variable
  intCount integer;
begin
  if (new.ID_RODZAJ is not null) then begin
    select
      count (*)
    from
      FK_S_CPV_RODZAJ scpvrodz
    where
      scpvrodz.ID = new.ID_RODZAJ
      and (scpvrodz.DATA_KONC < new.DATA_POCZ or scpvrodz.DATA_POCZ > new.DATA_KONC)
    into :intCount;
    if (intCount > 0) then exception NIEPRAWIDLOWY_RODZAJ_CPV;
  end
end^

create or alter trigger SCPV_RODZ_TRIG_U for FK_S_CPV
active before update position 3
as
declare variable
  intCount integer;
begin
  if (new.ID_RODZAJ is not null) then begin
    if ((new.ID_RODZAJ <> old.ID_RODZAJ) or (old.ID_RODZAJ is null)) then begin
      select
        count (*)
      from
        FK_S_CPV_RODZAJ scpvrodz
      where
        scpvrodz.ID = new.ID_RODZAJ
        and (scpvrodz.DATA_KONC < new.DATA_POCZ or scpvrodz.DATA_POCZ > new.DATA_KONC)
      into :intCount;
      if (intCount > 0) then exception NIEPRAWIDLOWY_RODZAJ_CPV;
    end
  end
end^

set term ; ^

/* ponizsza linia komentarza jest interpretowana przez exe jako instrukcja wykonania polecen wpisanych ponad nia: */
/*--------------------------------------------------------------- */

create or alter exception NIEPRAWIDLOWA_POZYCJA_CPV 'Okres wanoci pozycji CPV niezgodny z dat dokumentu!';
set term ^ ;

CREATE trigger CPV_SCPV_DOKUM_I for FK_CPV
active before insert position 1
as
declare variable
  intCount integer;
begin
  select
    count (*)
  from
    FK_DOKUMENTY dokum,
    FK_S_CPV scpv
  where
    dokum.ID = new.ID_DOKUMENT and
    scpv.ID = new.ID_S_CPV and
    dokum.DATA_DOK not between scpv.DATA_POCZ and scpv.DATA_KONC
  into :intCount;
  if (intCount > 0) then exception NIEPRAWIDLOWA_POZYCJA_CPV;
end^

create or alter trigger CPV_SCPV_DOKUM_U for FK_CPV
active before update position 1
as
declare variable
  intCount integer;
begin
  if ((new.ID_DOKUMENT <> old.ID_DOKUMENT) or (new.ID_S_CPV <> old.ID_S_CPV)) then begin
    select
      count (*)
    from
      FK_DOKUMENTY dokum,
      FK_S_CPV scpv
    where
      dokum.ID = new.ID_DOKUMENT and
      scpv.ID = new.ID_S_CPV and
      dokum.DATA_DOK not between scpv.DATA_POCZ and scpv.DATA_KONC
    into :intCount;
    if (intCount > 0) then exception NIEPRAWIDLOWA_POZYCJA_CPV;
  end
end^

set term ; ^

/* ponizsza linia komentarza jest interpretowana przez exe jako instrukcja wykonania polecen wpisanych ponad nia: */
/*--------------------------------------------------------------- */


